@@ -2,7 +2,7 @@
 ##
 ## Simple IP-Monitor Tools for TKINED.
 ##
-## Copyright (c) 1993, 1994
+## Copyright (c) 1993, 1994, 1995
 ##
 ## J. Schoenwaelder
 ## TU Braunschweig, Germany
@@ -55,9 +55,24 @@ proc CreateChart {id x y} {
 ##
 
 proc save { cmd } {
-    set rlist [ined restart]
-    lappend rlist $cmd
-    ined restart $rlist
+    set cmdList [ined restart]
+    if {[lsearch $cmdList $cmd] >= 0} return
+    lappend cmdList $cmd
+    ined restart $cmdList
+}
+
+##
+## Remove a command from the list of commands saved in the tkined map.
+##
+
+proc forget { cmd } {
+    set cmdList ""
+    foreach c [ined restart] {
+	if {$c != $cmd} {
+	    lappend cmdList $c
+	}
+    }
+    ined restart $cmdList
 }
 
 ##
@@ -69,12 +84,12 @@ proc restart { cmd args } {
 
     global interval
 
-    if {[regexp {^[0-9]*$} [lindex $args 0]]} {
+    if {! [catch {expr [lindex $args 0] + 0}]} {
 	set itv  [lindex $args 0]
 	set jid  [lindex $args 1]
 	set args [lrange $args 2 end]
 	set old_interval $interval
-	set interval $itv
+	set interval [expr int($itv)]
     }
 
     set ids $args
@@ -167,7 +182,7 @@ proc reachability {ids} {
     }
 	
     if {$ip_list != ""} {
-	set icmp_result [icmp $ip_list]
+	set icmp_result [icmp echo $ip_list]
 	set i 0
 	foreach id $id_list {
 	    set rtt [lindex [lindex $icmp_result $i] 1]
@@ -178,11 +193,11 @@ proc reachability {ids} {
 		    set reachability_label($id) [ined -noupdate label $id]
 		    ined -noupdate color $id red
 		}
-		ined flash $id [expr {[job interval [job current]] / 1000}]
+		ined flash $id [expr {[[job current] interval] / 1000}]
 		ined -noupdate attribute $id \
 		    "round trip time" "[ined name $id]\nunreachable"
 		ined -noupdate label $id "round trip time"
-		# syslog warning "[ined name $id] \[$ip\] unreachable"
+		MoJoAction $id "[ined name $id] \[$ip\] unreachable"
 	    } else {
 		if {$reachability_color($id) != ""} {
 		    ined -noupdate color $id $reachability_color($id)
@@ -257,17 +272,18 @@ proc pingtime {ids} {
     }
 	
     if {$ip_list != ""} {
-	set icmp_result [icmp $ip_list]
+	set icmp_result [icmp echo $ip_list]
 	set i 0
 	foreach id $id_list {
+	    set ip  [lindex [lindex $icmp_result $i] 0]
 	    set rtt [lindex [lindex $icmp_result $i] 1]
 	    if {$rtt < 0} {
 		ined values $id 0
 		ined -noupdate attribute $id \
 		    "round trip time" "[ined name $id]\nunreachable"
-		# syslog warning "[ined name $id] \[$ip\] unreachable"
+		MoJoAction $id "[ined name $id] \[$ip\] unreachable"
 	    } else {
-		ined values $id [expr {$rtt/10}]
+		ined values $id [expr {$rtt/10.0}]
 		ined -noupdate attribute $id \
 		    "round trip time" "rtt $rtt ms"
 		set txt "[ined name $id] \[$ip\] round trip time"
@@ -322,18 +338,24 @@ proc ntpoffset {ids} {
     foreach id $ids {
 	set ip $ntp_offset_ip($id)
 	
-	if {[catch {ntp $ip nase} err]} {
+	if {[catch {ntp $ip ntp} err]} {
 	    ined -noupdate attribute $id "ntp offset" "$err"
 	    ined values $id 0
 	} else {
-	    ined -noupdate attribute $id "ntp offset" "$nase(peer.offset) ms"
-	    ined -noupdate attribute $id "ntp peer" \
-		"[nslook $nase(peer.srcadr)]"
-
-	    set val [expr abs ($nase(peer.offset))]
-	    ined values $id $val
-	    set txt "[ined name $id] \[$ip\] ntp offset"
-	    MoJoCheckThreshold $id $txt $val "ms"
+	    if {[info exists ntp(peer.offset)]} {
+		set offset $ntp(peer.offset)
+		ined -noupdate attribute $id "ntp offset" "$offset ms"
+		set val [expr abs($offset)]
+		ined values $id $val
+		set txt "[ined name $id] \[$ip\] ntp offset"
+		MoJoCheckThreshold $id $txt $val "ms"
+	    }
+	    if {[info exists ntp(peer.srcadr)]} {
+		set srcadr [nslook $ntp(peer.srcadr)]
+		ined -noupdate attribute $id "ntp peer" $srcadr"
+	    } else {
+		ined -noupdate attribute $id "ntp offset" "no ntp peer"
+	    }
 	}
     }
 }
@@ -420,7 +442,10 @@ proc "System Load" {list} {
 	set id [lindex $id_ip 0]
 	set ip [lindex $id_ip 1]
 
-	if {[catch {sunrpc stat $ip} res]} continue
+	if {[catch {sunrpc stat $ip} res]} {
+	    ined acknowledge "[ined name $id] \[$ip\]: $res"
+	    continue
+	}
 
         set id [CreateChart $id 35 25]
 	catch {ined -noupdate scale $id 1}
@@ -498,7 +523,10 @@ proc "CPU Activity" {list} {
 	set id [lindex $id_ip 0]
 	set ip [lindex $id_ip 1]
 
-	if {[catch {sunrpc stat $ip} res]} continue
+	if {[catch {sunrpc stat $ip} res]} {
+	    ined acknowledge "[ined name $id] \[$ip\]: $res"
+	    continue
+	}
 
 	if {[ined type $id] != "BARCHART"} {
 	    set id [CloneNode $id [ined create BARCHART] 35 -35]
@@ -574,7 +602,10 @@ proc "Disk Activity" {list} {
 	set id [lindex $id_ip 0]
 	set ip [lindex $id_ip 1]
 
-	if {[catch {sunrpc stat $ip} res]} continue
+	if {[catch {sunrpc stat $ip} res]} {
+	    ined acknowledge "[ined name $id] \[$ip\]: $res"
+	    continue
+	}
 
         if {[ined type $id] != "BARCHART"} {
 	    set id [CloneNode $id [ined create BARCHART] -35 40]
@@ -652,7 +683,10 @@ proc "Interface Activity" {list} {
 	set id [lindex $id_ip 0]
 	set ip [lindex $id_ip 1]
 
-	if {[catch {sunrpc stat $ip} res]} continue
+	if {[catch {sunrpc stat $ip} res]} {
+	    ined acknowledge "[ined name $id] \[$ip\]: $res"
+	    continue
+	}
 
         if {[ined type $id] != "BARCHART"} {
 	    set id [CloneNode $id [ined create BARCHART] -35 -40]
@@ -706,8 +740,8 @@ proc etherload {ids} {
 	    [format "%3.2f MBit/s\\n%4.1f Packet/s\\n %4.1f Bytes/Packet" \
 	     $mbits_per_s $packets_per_s $bytes_per_s]
 	set load [list [expr {$mbits_per_s*12.5}] \
-		       [expr {$packets_per_s/20}] \
-		       [expr {$bytes_per_s/10}] ]
+		       [expr {$packets_per_s/20.0}] \
+		       [expr {$bytes_per_s/10.0}] ]
 	ined values $id $load
 	set txt "[ined name $id] \[$ip\] ethernet load"
 	MoJoCheckThreshold $id $txt $load
@@ -728,10 +762,12 @@ proc "Ethernet Load" {list} {
 	set id [lindex $id_ip 0]
 	set ip [lindex $id_ip 1]
 
-	if {[catch {sunrpc ether $ip open}]} {
+	if {[catch {sunrpc ether $ip open} res]} {
 	    if {[ined type $id] != "BARCHART"} {
 		ined -noupdate attribute $id ""
             }
+	    ined acknowledge "[ined name $id] \[$ip\]: $res"
+            continue
 	} else {
 
 	    if {[ined type $id] != "BARCHART"} {
@@ -777,6 +813,7 @@ proc users {ids} {
 	    if {$user == ""} continue
 	    if {[string match "Login*Name*" $user]} continue
 	    if {[string match "No*one*" $user]} continue
+	    if {[string match "*TTY*" $user]} continue
 	    set user [lindex $user 0]
 	    if {[info exists users($user)]} {
 		incr users($user)
@@ -787,13 +824,14 @@ proc users {ids} {
 	tcp close $f
 
 	set txt ""
-	if {[info exists users]} {
-	    foreach user [array names users] {
-		lappend txt "$user/$users($user)"
-	    }
+	foreach user [array names users] {
+	    lappend txt "$user/$users($user)"
+	}
+	if {$txt == ""} {
+	    set txt "no users"
 	}
 
-	ined -noupdate text $id [join $txt "\\n"]
+	ined -noupdate attribute $id "active users" $txt
     }
 }
 
@@ -809,10 +847,8 @@ proc "Active Users" { list } {
 	if {[catch {tcp connect $ip finger} f]} continue
 	tcp close $f
 
-	set size [ined -noupdate size $id]
-	set dx [expr {([lindex $size 2] - [lindex $size 0]) / 2 + 5}]
-	set dy [expr {([lindex $size 3] - [lindex $size 1]) / -2}]
-	set id [CloneNode $id [ined create TEXT ""] $dx $dy]
+	ined -noupdate attribute $id "active users" "no users"
+	ined label $id "active users"
 
 	set users_ip($id) $ip
 
@@ -852,14 +888,11 @@ proc "Set Parameter" {list} {
     global interval
     global default
 
-    set action [MoJoThresholdAction]
-
     set result [ined request "Monitor Parameter" \
         [list [list "# of ICMP retries:" $icmp_retries scale 1 10] \
           [list "ICMP timeout \[s\]:" $icmp_timeout scale 1 42] \
           [list "Delay between ICMP packets \[ms\]:" $icmp_delay scale 1 100] \
 	  [list "Interval \[s\]:" $interval entry 8] \
-	  [list "Threshold action:" $action check syslog flash write] \
 	  [list "Use Graph Diagram:" $default(graph) radio true false ] ] \
 	[list "set values" cancel] ]
 
@@ -869,10 +902,7 @@ proc "Set Parameter" {list} {
     set icmp_timeout   [lindex $result 2]
     set icmp_delay     [lindex $result 3]
     set interval       [lindex $result 4]
-    set action         [lindex $result 5]
-    set default(graph) [lindex $result 6]
-
-    MoJoThresholdAction $action
+    set default(graph) [lindex $result 5]
 
     if {$interval<1} { set interval 1 }
 
@@ -896,6 +926,14 @@ proc "Show Defaults" {list} {
 proc "Help IP-Monitor" {list} {
 
     ined browse "Help about IP-Monitor" {
+	"The IP-Monitor contains a set of utilities to monitor hosts" 
+	"using simple ICMP messages or rstat/etherd RPCs. You can define" 
+	"thresholds by setting the attributes Monitor:RisingThreshold" 
+	"or Monitor:FallingThreshold. The attribute Monitor:ThresholdAction" 
+	"controls what action will be taken if the rising threshold or" 
+	"the falling threshold is exceeded. The action types are syslog," 
+	"flash and write." 
+	"" 
 	"Check Reachability:" 
 	"    This command will periodically send an echo ICMP request to the" 
 	"    selected hosts and takes action when a host gets unreachable." 
@@ -929,7 +967,9 @@ proc "Help IP-Monitor" {list} {
 	"" 
 	"Active Users:" 
 	"    Display the number of active users as reported by the finger" 
-	"    daemon in a text object." 
+	"    daemon in the 'active users' attribute. Note, there might be" 
+	"    active users that are not displayed because finger output" 
+	"    is quite untrustworthy." 
 	"" 
 	"Monitor Job Info:" 
 	"    This command display information about all monitoring jobs" 
@@ -962,7 +1002,7 @@ proc "Delete IP-Monitor" {list} {
     global menus
     global reachability_ids
 
-    if {[job list] != ""} {
+    if {[job info] != ""} {
 	set res [ined confirm "Kill running monitoring jobs?" \
 		 [list "kill & exit" cancel] ]
 	if {$res == "cancel"} return
