@@ -2,7 +2,7 @@
 ##
 ## Simple Troubleshooting interpreter for [TK]INED.
 ##
-## Copyright (c) 1993, 1994
+## Copyright (c) 1993, 1994, 1995
 ##
 ## J. Schoenwaelder
 ## TU Braunschweig, Germany
@@ -29,9 +29,9 @@ proc netping {network} {
     if {[regexp "^\[0-9\]+\.\[0-9\]+\.\[0-9\]+$" $network]>0} {
 	set hosts ""
 	for {set a4 1} {$a4<255} {incr a4} {
-	    set hosts "$hosts $network.$a4"
+	    lappend hosts $network.$a4
 	}
-	if {[catch {icmp -timeout 15 $hosts} res]} {
+	if {[catch {icmp -timeout 15 echo $hosts} res]} {
 	    set res ""
 	}
 	return $res
@@ -65,9 +65,9 @@ proc Ping {list} {
 	    set ip [GetIpAddress $comp]
 	    if {$ip==""} {
 		ined acknowledge "Can not lookup IP Address for $host."
-		continue;
+		continue
 	    }
-	    if {[catch {icmp -size 0 -timeout 5 $ip} rtt]} {
+	    if {[catch {icmp -size 0 -timeout 5 echo $ip} rtt]} {
 		writeln "Could not send icmp packet: $rtt"
 		writeln
 		return
@@ -119,17 +119,17 @@ proc Ping {list} {
 proc "Multi Ping" {list} {
     static increment
     static maxsize
-
-    if ![info exists increment] { set increment "128" }
-    if ![info exists maxsize]   { set maxsize "2048" }
-
+    
+    if ![info exists increment] { set increment 128 }
+    if ![info exists maxsize]   { set maxsize 2048 }
+    
     set result [ined request "Send icmp Packets with varying sizes." \
-		[list [list "Increment \[bytes\]:" $increment scale 1 256] \
-		  [list "Max. packet size \[bytes\]:" $maxsize scale 64 2048] \
-		] [list start cancel] ]
-
+	    [list [list "Increment \[bytes\]:" $increment scale 1 512] \
+	    [list "Max. packet size \[bytes\]:" $maxsize scale 64 2048] \
+	    ] [list start cancel] ]
+    
     if {[lindex $result 0] == "cancel"} return
-
+    
     set increment [lindex $result 1]
     set maxsize   [lindex $result 2]
 
@@ -143,7 +143,7 @@ proc "Multi Ping" {list} {
             }
 	    writeln "Multi ping for $host \[$ip\]:"
 	    for {set size 0} {$size <= $maxsize} {incr size $increment} {
-		if {[catch {icmp -size $size $ip} time]} {
+		if {[catch {icmp -size $size echo $ip} time]} {
 		    writeln "Could not send icmp packet: $time"
 		    writeln
 		    return
@@ -177,7 +177,7 @@ proc "Trace Route" {list} {
 	    write [format "%2d  " $ttl]
 	    set last_ip ""
 	    for {set i 0} {$i < 3} {incr i} {
-		if {[catch {icmp -trace $ttl $ip} trace]} {
+		if {[catch {icmp trace $ttl $ip} trace]} {
 		    writeln "Could not send icmp packet: $trace"
 		    writeln
 		    return
@@ -215,7 +215,7 @@ proc "Trace Route" {list} {
 
 proc "Netmask" {list} {
     ForeachIpNode id ip host $list {
-	if {[catch {icmp -mask $ip} result]} {
+	if {[catch {icmp mask $ip} result]} {
 	    writeln "Could not send icmp packet: $result"
 	    writeln
 	    return
@@ -244,7 +244,12 @@ proc Telnet {list} {
     }
     
     ForeachIpNode id ip host $list {
-	catch {exec $xterm -title $host -e telnet $ip &}
+	set port [ined attribute $id IP-Trouble:TelnetPort]
+	if {$port != ""} {
+	    catch {exec $xterm -title $host -e telnet $ip $port &}
+	} else {
+	    catch {exec $xterm -title $host -e telnet $ip &}
+	}
     }
 }
 
@@ -320,7 +325,7 @@ proc "DNS Info" {list} {
     ForeachIpNode id ip host $list {
 	if {[catch {dns ptr $ip} name]} { set name $host }
         set name [lindex $name 0]
-	if {[catch {dns a $name} a]}             { set a "" }
+	if {[catch {dns address $name} a]}       { set a "" }
 	if {[catch {dns ptr [lindex $a 0]} ptr]} { set ptr "" }
 	if {[catch {dns hinfo $name} hinfo]}     { set hinfo "" }
 	if {[catch {dns mx $name} mx]}           { set mx "" }
@@ -434,40 +439,34 @@ proc "NTP Info" { list } {
 }
 
 ##
-## Scan the /etc/services file and try to connect to all ports
+## Scan the services file and try to connect to all ports
 ## listed there. This will not really tell us, if we could you
 ## a service but it may be a hint.
 ##
 
 proc "TCP Services" {list} {
 
-    if {[catch {open /etc/services r} s]} {
-	ined acknowledge "Can not open /etc/services."
-	return
-    }
-
     ForeachIpNode id ip host $list {
 	writeln "TCP services on $host \[$ip\]:"
-	seek $s 0
-	set service "X11  6000/tcp"
-	while {! [eof $s]} {
-	    if {[regexp "\[0-9\]*/tcp" $service]>0} {
-		set name [lindex $service 0]
-		if {[catch {tcp connect $ip $name} f]} {
-		    set service [gets $s]
-		    continue
-		}
-		tcp shutdown $f all
-		tcp close $f
-		writeln [format "  %-12s %8s %s" $name \
-			 [lindex $service 1] [lrange $service 2 end]]
+
+	foreach service [netdb services] {
+	    if {[lindex $service 2] == "tcp"} {
+		lappend serviceList $service
 	    }
-	    set service [gets $s]
+	}
+	lappend serviceList "X11 6000 tcp"
+	foreach serv $serviceList {
+	    set name [lindex $serv 0]
+	    if {[catch {tcp connect $ip $name} f]} {
+		continue
+	    }
+	    tcp shutdown $f all
+	    tcp close $f
+	    writeln [format "  %-12s %6s/%s" \
+		     $name [lindex $serv 1] [lindex $serv 2]]
 	}
 	writeln
     }
-
-    close $s
 }
 
 ##
@@ -581,7 +580,7 @@ proc rstat_diff {l1 l2 period} {
 
 proc rstat {host period} {
     set stat [sunrpc stat $host]
-    sleep $period
+    after [expr $period * 1000]
     set newstat [sunrpc stat $host]
     return [rstat_diff $newstat $stat $period]
 }
@@ -631,7 +630,7 @@ proc "ether RPC" {list} {
     ForeachIpNode id ip host $list {
 	if {[catch {
 	    sunrpc ether $ip open
-	    sleep $interval
+	    after [expr $interval * 1000]
 	    set res [sunrpc ether $ip]
 	    sunrpc ether $ip close
 	    set result ""
@@ -658,12 +657,11 @@ proc "Set Parameter" {list} {
     global icmp_retries icmp_timeout icmp_delay
     global icmp_routelength
 
-
     set result [ined request "IP Trouble Parameter" \
 	[list [list "# of ICMP retries:" $icmp_retries scale 1 10] \
           [list "ICMP timeout \[s\]:" $icmp_timeout scale 1 42] \
           [list "Delay between ICMP packets \[ms\]:" $icmp_delay scale 1 100] \
-          [list "Max. length of a route:" $icmp_routelength scale 1 30] ] \
+          [list "Max. length of a route:" $icmp_routelength scale 1 42] ] \
 	[list "set values" cancel] ]
 
     if {[lindex $result 0] == "cancel"} return
@@ -699,7 +697,9 @@ proc "Help IP-Trouble" {list} {
 	"    Trace the route to the selected nodes." 
 	"" 
 	"Telnet:" 
-	"    Open a telnet session to the selected nodes." 
+	"    Open a telnet session to the selected nodes. You can" 
+	"    define an alternate port number by setting the attribute" 
+	"    IP-Trouble:TelnetPort." 
 	"" 
 	"Rlogin:" 
 	"    Open a rlogin session to the selected nodes." 
