@@ -2,7 +2,7 @@
 ##
 ## Simple SNMP-Troubleshooting utilities for tkined.
 ##
-## Copyright (c) 1993, 1994
+## Copyright (c) 1993, 1994, 1995
 ##
 ## J. Schoenwaelder
 ## TU Braunschweig, Germany
@@ -26,9 +26,9 @@ SnmpInit SNMP-Trouble
 ## all mib modules and reduce memory requirements and statup time.
 ##
 
-mib load $scotty_lib/mibs/rfc1447.mib
-mib load $scotty_lib/mibs/rfc1450.mib
-mib load $scotty_lib/mibs/rfc1451.mib
+# mib load rfc1447.mib
+# mib load rfc1450.mib
+# mib load rfc1451.mib
 
 ##
 ## List all scalars of a group for the node objects in list.
@@ -36,7 +36,9 @@ mib load $scotty_lib/mibs/rfc1451.mib
 
 proc ShowScalars {list group} {
     ForeachIpNode id ip host $list {
-	SnmpShowScalars $ip $group
+	set s [SnmpOpen $id $ip]
+        SNMP_ShowScalars $s $group
+        $s destroy
     }
 }
 
@@ -46,7 +48,9 @@ proc ShowScalars {list group} {
 
 proc ShowTable {list table} {
     ForeachIpNode id ip host $list {
-	SnmpShowTable $ip $table
+	set s [SnmpOpen $id $ip]
+        SNMP_ShowTable $s $table
+        $s destroy
     }
 }
 
@@ -54,23 +58,26 @@ proc ShowTable {list table} {
 ## Try to figure out if the devices responds to snmp requests.
 ##
 
-proc SnmpDeviceCallback {host ip s args} {
-    if {$args == "" } {
+proc SnmpDeviceCallback {id ip host session error} {
+    if {$error == "noError" } {
 	write   [format "%-32s \[" $host ]
 	write   $ip "IpFlash $ip"
 	writeln "\] "
+	ined -noupdate attribute $id "SNMP:Config" [$session configure]
     } else {
-	writeln [format "%-32s \[%s\] %s" $host $ip $args]
+	writeln [format "%-32s \[%s\] %s" $host $ip $error]
     }
-    SnmpClose $s
+    $session destroy
 }
 
 proc "SNMP Devices" {list} {
     writeln "SNMP devices:"
     ForeachIpNode id ip host $list {
-	[SnmpOpen $ip] getnext 1.0 [list SnmpDeviceCallback $host $ip %S] \
-		[list SnmpDeviceCallback $host $ip %S %E]
+	[SnmpOpen $id $ip] getnext 1.0 \
+		[list SnmpDeviceCallback $id $ip $host %S %E]
     }
+    snmp wait
+    writeln
 }
 
 ##
@@ -85,62 +92,26 @@ proc "System Information" {list} {
 ## Get the list of interfaces and their status.
 ##
 
-proc "Status" {list} {
+proc "IF Status" {list} {
     ForeachIpNode id ip host $list {
-	set s [SnmpOpen $ip]
+	set s [SnmpOpen $id $ip]
 	set txt "Interface status of $host \[$ip\]:\n"
 	append txt "ifIndex ifDescr            ifAdminStatus ifOperStatus (ifType)\n"
-	$s walk x "ifIndex ifDescr ifAdminStatus ifOperStatus ifType" {
-	    set ifIndex [lindex [lindex $x 0] 2]
-	    set ifDescr [lindex [lindex [lindex [lindex $x 1] 2] 0] 0]
-	    set ifAdmin [lindex [lindex $x 2] 2]
-	    set ifOper  [lindex [lindex $x 3] 2]
-	    set ifType  [lindex [lindex $x 4] 2]
-	    append txt [format "%5s   %-16s %12s %12s    (%s)\n" \
+	try {
+	    $s walk x "ifIndex ifDescr ifAdminStatus ifOperStatus ifType" {
+		set ifIndex [lindex [lindex $x 0] 2]
+		set ifDescr [lindex [lindex [lindex [lindex $x 1] 2] 0] 0]
+		set ifAdmin [lindex [lindex $x 2] 2]
+		set ifOper  [lindex [lindex $x 3] 2]
+		set ifType  [lindex [lindex $x 4] 2]
+		append txt [format "%5s   %-16s %12s %12s    (%s)\n" \
 			$ifIndex $ifDescr $ifAdmin $ifOper $ifType]
-	}
-	SnmpClose $s
-	writeln $txt
-    }
-}
-
-##
-## Get the IP addresses belonging to the interfaces.
-##
-
-proc "IP Address" {list} {
-    ForeachIpNode id ip host $list {
-	set s [SnmpOpen $ip]
-	set txt "Interface IP address of $host \[$ip\]:\n"
-	append txt "ifIndex                 ifType        ipAddress\n"
-	$s walk x "ipRouteDest ipRouteNextHop ipRouteIfIndex ipRouteMetric1" {
-	    set dest [lindex [lindex $x 0] 2]
-	    set next [lindex [lindex $x 1] 2]
-	    set ifin [lindex [lindex $x 2] 2]
-	    set metr [lindex [lindex $x 3] 2]
-	    if {($ifin > 0) && ($metr == 0) 
-	    && ($dest != "0.0.0.0") && ($next != "0.0.0.0")} {
-		if {![info exists ips($ifin)]} {
-		    set ips($ifin) $next
-		} elseif {[lsearch $ips($ifin) $next] < 0} {
-		    lappend ips($ifin) $next
-		}
 	    }
+	} msg { 
+	    append txt "$msg\n" 
 	}
-	
-	if {[info exists ips]} {
-	    foreach ifin [lsort -integer [array names ips]] {
-		if {[catch {$s get ifType.$ifin} iftype]} {
-		    set iftype ""
-		}
-		set iftype [lindex [lindex $iftype 0] 2]
-		append txt [format "%5s %30s  %s\n" $ifin $iftype $ips($ifin)]
-	    }
-	    unset ips
-	}
-	
-	SnmpClose $s
 	writeln $txt
+	$s destroy
     }
 }
 
@@ -148,22 +119,26 @@ proc "IP Address" {list} {
 ## Get the list of interfaces and their status.
 ##
 
-proc "Parameter" {list} {
+proc "IF Parameter" {list} {
     ForeachIpNode id ip host $list {
-	set s [SnmpOpen $ip]
+	set s [SnmpOpen $id $ip]
 	set txt "Interface parameter of $host \[$ip\]:\n"
 	append txt "ifIndex ifDescr              ifSpeed      ifMtu   ifPhysAddress\n"
-	$s walk x "ifIndex ifDescr ifSpeed ifMtu ifPhysAddress" {
-	    set ifIndex [lindex [lindex $x 0] 2]
-	    set ifDescr [lindex [lindex [lindex [lindex $x 1] 2] 0] 0]
-	    set ifSpeed [lindex [lindex $x 2] 2]
-	    set ifMtu   [lindex [lindex $x 3] 2]
-	    set ifPhys  [lindex [lindex $x 4] 2]
-	    append txt [format " %5s  %-16s %11s %10s   %s\n" \
+	try {
+	    $s walk x "ifIndex ifDescr ifSpeed ifMtu ifPhysAddress" {
+		set ifIndex [lindex [lindex $x 0] 2]
+		set ifDescr [lindex [lindex [lindex [lindex $x 1] 2] 0] 0]
+		set ifSpeed [lindex [lindex $x 2] 2]
+		set ifMtu   [lindex [lindex $x 3] 2]
+		set ifPhys  [lindex [lindex $x 4] 2]
+		append txt [format " %5s  %-16s %11s %10s   %s\n" \
 			$ifIndex $ifDescr $ifSpeed $ifMtu $ifPhys]
+	    }
+	} msg { 
+	    append txt "$msg\n" 
 	}
-	SnmpClose $s
 	writeln $txt
+	$s destroy
     }
 }
 
@@ -171,48 +146,56 @@ proc "Parameter" {list} {
 ## We should check which interfaces are up and running.
 ##
 
-proc "Usage Statistics" {list} {
+proc "IF Usage Statistics" {list} {
     ForeachIpNode id ip host $list {
-	set s [SnmpOpen $ip]
+	set s [SnmpOpen $id $ip]
 	set txt "Interface usage of $host \[$ip\]:\n"
         append txt "ifIndex ifDescr           ifInOctets  ifInUcastPkts  ifOutOctets ifOutUcastPkts\n"
-	$s walk x "ifIndex ifDescr ifInOctets ifInUcastPkts \
-		   ifOutOctets ifOutUcastPkts" {
-	    set ifIndex   [lindex [lindex $x 0] 2]
-	    set ifDescr   [lindex [lindex [lindex [lindex $x 1] 2] 0] 0]
-	    set ifInOct   [lindex [lindex $x 2] 2]
-	    set ifInUOct  [lindex [lindex $x 3] 2]
-	    set ifOutOct  [lindex [lindex $x 4] 2]
-	    set ifOutUOct [lindex [lindex $x 5] 2]
-	    append txt [format " %5s  %-15s %12s %14s %12s %14s\n" \
+	try {
+	    $s walk x "ifIndex ifDescr ifInOctets ifInUcastPkts \
+		    ifOutOctets ifOutUcastPkts" {
+		set ifIndex   [lindex [lindex $x 0] 2]
+		set ifDescr   [lindex [lindex [lindex [lindex $x 1] 2] 0] 0]
+		set ifInOct   [lindex [lindex $x 2] 2]
+		set ifInUOct  [lindex [lindex $x 3] 2]
+		set ifOutOct  [lindex [lindex $x 4] 2]
+		set ifOutUOct [lindex [lindex $x 5] 2]
+		append txt [format " %5s  %-15s %12s %14s %12s %14s\n" \
 			$ifIndex $ifDescr $ifInOct $ifInUOct \
-			    $ifOutOct $ifOutUOct]
-	}
-	SnmpClose $s
+			$ifOutOct $ifOutUOct]
+	    }
+	} msg {
+            append txt "$msg\n"
+        }
 	writeln $txt
+	$s destroy
     }
 }
 
 
-proc "Error Statistics" {list} {
+proc "IF Error Statistics" {list} {
     ForeachIpNode id ip host $list {
-	set s [SnmpOpen $ip]
+	set s [SnmpOpen $id $ip]
 	set txt "Interface errors of $host \[$ip\]:\n"
 	append txt "ifIndex ifDescr           ifInErrors   ifInDiscards  ifOutErrors  ifOutDiscards\n"
-	$s walk x "ifIndex ifDescr ifInErrors ifInDiscards \
-		   ifOutErrors ifOutDiscards" {
-	    set ifIndex   [lindex [lindex $x 0] 2]
-	    set ifDescr   [lindex [lindex [lindex [lindex $x 1] 2] 0] 0]
-	    set ifInErr   [lindex [lindex $x 2] 2]
-	    set ifInDisc  [lindex [lindex $x 3] 2]
-	    set ifOutErr  [lindex [lindex $x 4] 2]
-	    set ifOutDisc [lindex [lindex $x 5] 2]
-	    append txt [format " %5s  %-15s %12s %14s %12s %14s\n" \
+	try {
+	    $s walk x "ifIndex ifDescr ifInErrors ifInDiscards \
+		    ifOutErrors ifOutDiscards" {
+		set ifIndex   [lindex [lindex $x 0] 2]
+		set ifDescr   [lindex [lindex [lindex [lindex $x 1] 2] 0] 0]
+		set ifInErr   [lindex [lindex $x 2] 2]
+		set ifInDisc  [lindex [lindex $x 3] 2]
+		set ifOutErr  [lindex [lindex $x 4] 2]
+		set ifOutDisc [lindex [lindex $x 5] 2]
+		append txt [format " %5s  %-15s %12s %14s %12s %14s\n" \
 			$ifIndex $ifDescr $ifInErr $ifInDisc \
-			    $ifOutErr $ifOutDisc]
-	}
-	SnmpClose $s
+			$ifOutErr $ifOutDisc]
+	    }
+        } msg {
+            append txt "$msg\n"
+        }
 	writeln $txt
+	$s destroy
     }
 }
 
@@ -220,42 +203,46 @@ proc "Error Statistics" {list} {
 ## Display relative error statistics.
 ##
 
-proc "Quality" {list} {
+proc "IF Quality" {list} {
     ForeachIpNode id ip host $list {
-	set s [SnmpOpen $ip]
+	set s [SnmpOpen $id $ip]
 	set txt "Interface quality of $host \[$ip\]:\n"
 	append txt "(may be invalid if counter have wrapped)\n"
 	append txt "ifIndex ifDescr        Error Rate   Discard Rate\n"
-	$s walk x "ifIndex ifDescr ifInUcastPkts ifInNUcastPkts \
-		   ifOutUcastPkts ifOutNUcastPkts ifInErrors ifOutErrors \
-		   ifInDiscards ifOutDiscards" {
-	    set ifIndex   [lindex [lindex $x 0] 2]
-	    set ifDescr   [lindex [lindex [lindex [lindex $x 1] 2] 0] 0]
-	    set in  [expr [lindex [lindex $x 2] 2] + [lindex [lindex $x 3] 2]]
-	    set out [expr [lindex [lindex $x 4] 2] + [lindex [lindex $x 5] 2]]
-	    set pkts [expr {$in + $out}]
-	    if {$in == 0} {
-		set err 0
-		set dis 0
-	    } else {
-		set err [expr {[lindex [lindex $x 6] 2] \
-			+ [lindex [lindex $x 7] 2]}]
-		set err [expr {100 * double($err) / $pkts}]
+	try {
+	    $s walk x "ifIndex ifDescr ifInUcastPkts ifInNUcastPkts \
+		    ifOutUcastPkts ifOutNUcastPkts ifInErrors ifOutErrors \
+		    ifInDiscards ifOutDiscards" {
+		set ifIndex   [lindex [lindex $x 0] 2]
+		set ifDescr   [lindex [lindex [lindex [lindex $x 1] 2] 0] 0]
+		set in  [expr [lindex [lindex $x 2] 2] + [lindex [lindex $x 3] 2]]
+		set out [expr [lindex [lindex $x 4] 2] + [lindex [lindex $x 5] 2]]
+		set pkts [expr {$in + $out}]
+		if {$in == 0} {
+		    set err 0
+		    set dis 0
+		} else {
+		    set err [expr {[lindex [lindex $x 6] 2] \
+			    + [lindex [lindex $x 7] 2]}]
+		    set err [expr {100 * double($err) / $pkts}]
+		}
+		if {$out == 0} {
+		    set err 0
+		    set dis 0
+		} else {
+		    set dis [expr {[lindex [lindex $x 8] 2] \
+			    + [lindex [lindex $x 9] 2]}]
+		    set dis [expr {100 * double($dis) / $pkts}]
+		}
+		append txt \
+			[format "%5s   %-15s  %5.2f %%        %5.2f %%\n" \
+			$ifIndex $ifDescr $err $dis]
 	    }
-	    if {$out == 0} {
-		set err 0
-		set dis 0
-	    } else {
-		set dis [expr {[lindex [lindex $x 8] 2] \
-			+ [lindex [lindex $x 9] 2]}]
-		set dis [expr {100 * double($dis) / $pkts}]
-	    }
-	    append txt \
-		[format "%5s   %-15s  %5.2f %%        %5.2f %%\n" \
-		 $ifIndex $ifDescr $err $dis]
-	}
-	SnmpClose $s
+	} msg {
+            append txt "$msg\n"
+        }
 	writeln $txt
+	$s destroy
     }
 }
 
@@ -267,29 +254,62 @@ proc "IP Statistics" {list} {
     ShowScalars $list ip
 }
 
+##
+## Get the IP addresses belonging to this device.
+##
+
+proc "IP Addresses" {list} {
+    ForeachIpNode id ip host $list {
+	set s [SnmpOpen $id $ip]
+	set txt "IP addresses of $host \[$ip\]:\n"
+	append txt "ifIndex ipAdEntAddr     ipAdEntNetMask ipAdEntBcastAddr\n"
+	try {
+	    set list ""
+	    $s walk x {ipAdEntAddr ipAdEntNetMask 
+	               ipAdEntBcastAddr ipAdEntIfIndex} {
+		set ipAdEntAddr      [lindex [lindex $x 0] 2]
+		set ipAdEntNetMask   [lindex [lindex $x 1] 2]
+		set ipAdEntBcastAddr [lindex [lindex $x 2] 2]
+		set ipAdEntIfIndex   [lindex [lindex $x 3] 2]
+		lappend list [format "%5s   %-15s %-15s %5s" $ipAdEntIfIndex \
+			$ipAdEntAddr $ipAdEntNetMask $ipAdEntBcastAddr]
+	    }
+	    append txt "[join [lsort $list] "\n"]\n"
+	} msg { 
+	    append txt "$msg\n" 
+	}
+	writeln $txt
+	$s destroy
+    }
+}
+
 ##
 ## Show the routing tables of the agents.
 ##
 
 proc "IP Routing Table" {list} {
     ForeachIpNode id ip host $list {
-	set s [SnmpOpen $ip]
+	set s [SnmpOpen $id $ip]
 	set txt "Routing Table of $host \[$ip\]:\n"
 	append txt "ipRouteDest      ipRouteNextHop   ipRouteMask      IfIndex    Type     Proto\n"
-	$s walk x "ipRouteDest ipRouteNextHop ipRouteMask ipRouteIfIndex \
-		   ipRouteType ipRouteProto" {
-	    set ipRouteDest    [lindex [lindex $x 0] 2]
-	    set ipRouteNext    [lindex [lindex $x 1] 2]
-	    set ipRouteMask    [lindex [lindex $x 2] 2]
-	    set ipRouteIfIndex [lindex [lindex $x 3] 2]
-	    set ipRouteType    [lindex [lindex $x 4] 2]
-	    set ipRouteProto   [lindex [lindex $x 5] 2]
-	    append txt [format "%-16s %-16s %-16s %5d %10s %8s\n" \
+	try {
+	    $s walk x {ipRouteDest ipRouteNextHop ipRouteMask ipRouteIfIndex
+	    ipRouteType ipRouteProto} {
+		set ipRouteDest    [lindex [lindex $x 0] 2]
+		set ipRouteNext    [lindex [lindex $x 1] 2]
+		set ipRouteMask    [lindex [lindex $x 2] 2]
+		set ipRouteIfIndex [lindex [lindex $x 3] 2]
+		set ipRouteType    [lindex [lindex $x 4] 2]
+		set ipRouteProto   [lindex [lindex $x 5] 2]
+		append txt [format "%-16s %-16s %-16s %5d %10s %8s\n" \
 			$ipRouteDest $ipRouteNext $ipRouteMask \
-			    $ipRouteIfIndex $ipRouteType $ipRouteProto]
+			$ipRouteIfIndex $ipRouteType $ipRouteProto]
+	    }
+	} msg { 
+	    append txt "$msg\n" 
 	}
-	SnmpClose $s
 	writeln $txt
+	$s destroy
     }
 }
 
@@ -300,46 +320,46 @@ proc "IP Routing Table" {list} {
 proc "IP ARP Table" {list} {
     global vendor
     ForeachIpNode id ip host $list {
-	set s [SnmpOpen $ip]
+	set s [SnmpOpen $id $ip]
 	set txt "ARP Table of $host \[$ip\]:\n"
 	append txt "ifIndex PhysAddress        NetAddress         Type       Vendor\n"
 	set done 0
-	SnmpGetBulk $s {
-	    ipNetToMediaIfIndex
-	    ipNetToMediaPhysAddress
-	    ipNetToMediaNetAddress
-	    ipNetToMediaType
-	} x {
+	$s walk x {ipNetToMediaIfIndex ipNetToMediaPhysAddress 
+	ipNetToMediaNetAddress ipNetToMediaType} {
+	    set ipNetToMediaIfIndex	    [lindex [lindex $x 0] 2]
+	    set ipNetToMediaPhysAddress [lindex [lindex $x 1] 2]
+	    set ipNetToMediaNetAddress  [lindex [lindex $x 2] 2]
+	    set ipNetToMediaType        [lindex [lindex $x 3] 2]
 	    set done 1
-	    set l [join [lrange [split [lindex $x 1] :] 0 2] {}]
+	    set l [join [lrange [split $ipNetToMediaPhysAddress :] 0 2] {}]
 	    set l [string toupper $l]
 	    if {[info exists vendor($l)]} {
 		set vend $vendor($l)
 	    } else {
 		set vend ""
 	    }
-	    set aa "format { %5s  %-18s %-18s %-10s} [join $x]"
-	    append txt "[eval $aa] $vend\n"
+	    append txt [format " %5s  %-18s %-18s %-10s %s\n" \
+			$ipNetToMediaIfIndex $ipNetToMediaPhysAddress \
+			$ipNetToMediaNetAddress $ipNetToMediaType $vend]
 	}
 	if {!$done} {
-	    SnmpGetBulk $s {
-		atIfIndex
-		atPhysAddress
-		atNetAddress
-	    } x {
-		set l [join [lrange [split [lindex $x 1] :] 0 2] {}]
+	    $s walk x {atIfIndex atPhysAddress atNetAddress} {
+		set atIfIndex     [lindex [lindex $x 0] 2]
+		set atPhysAddress [lindex [lindex $x 1] 2]
+		set atNetAddress  [lindex [lindex $x 2] 2]
+		set l [join [lrange [split $atPhysAddress :] 0 2] {}]
 		set l [string toupper $l]
 		if {[info exists vendor($l)]} {
 		    set vend $vendor($l)
 		} else {
 		    set vend ""
 		}
-		set aa "format { %5s  %-18s %-18s } [join $x]"
-		append txt "[eval $aa] $vend\n"
+		append txt [format " %5s  %-18s %-18s %s\n" \
+			    $atIfIndex $atPhysAddress $atNetAddress $vend]
 	    }
 	}
-	SnmpClose $s
 	writeln $txt
+	$s destroy
     }
 }
 
@@ -357,11 +377,11 @@ proc "TCP Statistics" {list} {
 
 proc "TCP Connections" {list} {
     ForeachIpNode id ip host $list {
-	set s [SnmpOpen $ip]
+	set s [SnmpOpen $id $ip]
 	set percent -1
-	if {![catch {SnmpGet $s "tcp.tcpMaxConn.0 tcp.tcpCurrEstab.0"} res]} {
-	    set tcpMaxConn   [lindex $res 0]
-	    set tcpCurrEstab [lindex $res 1]
+	if {![catch {$s get "tcp.tcpMaxConn.0 tcp.tcpCurrEstab.0"} vbl]} {
+	    set tcpMaxConn   [lindex [lindex $vbl 0] 2]
+	    set tcpCurrEstab [lindex [lindex $vbl 1] 2]
 	    if {$tcpMaxConn > 0} {
 		set percent [expr {$tcpCurrEstab/double($tcpMaxConn)*100}]
 	    }
@@ -372,20 +392,33 @@ proc "TCP Connections" {list} {
 	} else {
 	    set txt "TCP Connections of $host \[$ip\]:\n"
 	}
-	append txt "State            LocalAddress LocalPort RemoteAddress RemotePort\n"
-	$s walk x "tcpConnState tcpConnLocalAddress tcpConnLocalPort \
-		   tcpConnRemAddress tcpConnRemPort" {
-	    set tcpConnState        [lindex [lindex $x 0] 2]
-	    set tcpConnLocalAddress [lindex [lindex $x 1] 2]
-	    set tcpConnLocalPort    [lindex [lindex $x 2] 2]
-	    set tcpConnRemAddress   [lindex [lindex $x 3] 2]
-	    set tcpConnRemPort      [lindex [lindex $x 4] 2]
-	    append txt [format "%-12s %16s %6s %16s %6s\n" $tcpConnState \
+	append txt "State            LocalAddress       LocalPort   RemoteAddress       RemotePort\n"
+	try {
+	    $s walk x "tcpConnState tcpConnLocalAddress tcpConnLocalPort \
+		    tcpConnRemAddress tcpConnRemPort" {
+		set tcpConnState        [lindex [lindex $x 0] 2]
+		set tcpConnLocalAddress [lindex [lindex $x 1] 2]
+		set tcpConnLocalPort    [lindex [lindex $x 2] 2]
+		set tcpConnRemAddress   [lindex [lindex $x 3] 2]
+		set tcpConnRemPort      [lindex [lindex $x 4] 2]
+
+		if {![catch {netdb services name "$tcpConnLocalPort tcp"} n]} {
+		    set tcpConnLocalPort "$n"
+		}
+		if {![catch {netdb services name "$tcpConnRemPort tcp"} n]} {
+		    set tcpConnRemPort "$n"
+		}
+
+		append txt [format "%-12s %16s %15s %16s %15s\n" \
+			$tcpConnState \
 			$tcpConnLocalAddress $tcpConnLocalPort \
-			    $tcpConnRemAddress $tcpConnRemPort]
+			$tcpConnRemAddress $tcpConnRemPort]
+	    }
+	} msg {
+	    append txt "$msg\n"
 	}
-	SnmpClose $s
 	writeln $txt
+	$s destroy
     }
 }
 
@@ -403,18 +436,24 @@ proc "UDP Statistics" {list} {
 
 proc "UDP Listener" {list} {
     ForeachIpNode id ip host $list {
-	set s [SnmpOpen $ip]
-
+	set s [SnmpOpen $id $ip]
 	set txt "UDP Listener of $host \[$ip\]:\n"
-	append txt "  LocalAddress       LocalPort\n"
-	$s walk x "udpLocalAddress udpLocalPort" {
-	    set udpLocalAddress	[lindex [lindex $x 0] 2]
-	    set udpLocalPort    [lindex [lindex $x 1] 2]
-	    append txt [format "  %-16s    %6s\n" \
+	append txt "      LocalAddress       LocalPort\n"
+	try {
+	    $s walk x "udpLocalAddress udpLocalPort" {
+		set udpLocalAddress	[lindex [lindex $x 0] 2]
+		set udpLocalPort    [lindex [lindex $x 1] 2]
+		if {![catch {netdb services name "$udpLocalPort udp"} n]} {
+		    set udpLocalPort "$n"
+		}
+		append txt [format "  %16s %15s\n" \
 			$udpLocalAddress $udpLocalPort]
+	    }
+	} msg {
+	    append txt "$msg\n"
 	}
-	SnmpClose $s
 	writeln $txt
+	$s destroy
     }
 }
 
@@ -440,10 +479,10 @@ proc "ICMP Statistics" {list} {
 
 proc "Parties" {list} {
     ForeachIpNode id ip host $list {
-	set s [SnmpOpen $ip]
+	set s [SnmpOpen $id $ip]
 	set txt "SNMPv2 Parties of $host \[$ip\]:\n"
 	append txt "Index  Transport-Address          AuthProtocol       PrivProtocol \n"
-	if {[catch {
+	try {
 	    $s walk x "partyIndex partyTDomain partyTAddress \
 		    partyAuthProtocol partyPrivProtocol" {
 		set Index    [lindex [lindex $x 0] 2]
@@ -459,12 +498,11 @@ proc "Parties" {list} {
 		append txt [format "%4s   %-20s %18s %18s\n" \
 			$Index $TAddress $AuthPro $PrivPro]
 	    }
-	} err]} {
-	    writeln $err
-	} else {
-	    writeln $txt
+	} msg {
+	    append txt "$msg\n"
 	}
-	SnmpClose $s
+	writeln $txt
+	$s destroy
     }
 }
 
@@ -474,10 +512,10 @@ proc "Parties" {list} {
 
 proc "Contexts" {list} {
     ForeachIpNode id ip host $list {
-        set s [SnmpOpen $ip]
+        set s [SnmpOpen $id $ip]
         set txt "SNMPv2 Contexts of $host \[$ip\]:\n"
         append txt "Index  ViewIndex Local       Storage \n"
-        if {[catch {
+        try {
 	    $s walk x "contextIndex contextViewIndex contextLocal \
 		    contextStorageType" {
 		set Index    [lindex [lindex $x 0] 2]
@@ -487,12 +525,11 @@ proc "Contexts" {list} {
 		append txt [format "%4s  %6s %8s %14s\n" \
                         $Index $ViewIdx $Local $Storage]
 	    }
-	} err]} {
-	    writeln $err
-	} else {
-	    writeln $txt
+	} msg {
+	    append txt "$msg\n"
 	}
-        SnmpClose $s
+	writeln $txt
+        $s destroy
     }
 }
 
@@ -502,10 +539,10 @@ proc "Contexts" {list} {
 
 proc "Views" {list} {
     ForeachIpNode id ip host $list {
-        set s [SnmpOpen $ip]
+        set s [SnmpOpen $id $ip]
         set txt "SNMPv2 Views of $host \[$ip\]:\n"
         append txt "Index Tree Mask Type Storage \n"
-        if {[catch {
+        try {
 	    $s walk x "viewIndex viewSubtree viewMask viewType viewStorageType" {
 		set Index    [lindex [lindex $x 0] 2]
 		set Tree     [lindex [lindex $x 1] 2]
@@ -515,12 +552,11 @@ proc "Views" {list} {
 		append txt [format "%4s  %6s %8s %14s %14s\n" \
                         $Index $Tree $Mask $Type $Storage]
 	    }
-	} err]} {
-	    writeln $err
-	} else {
-	    writeln $txt
+	} msg {
+	    append txt "$msg\n"
 	}
-        SnmpClose $s
+	writeln $txt
+        $s destroy
     }
 }
 
@@ -538,21 +574,57 @@ proc "Statistics" {list} {
 
 proc "Access Control" {list} {
     ForeachIpNode id ip host $list {
-        set s [SnmpOpen $ip]
+        set s [SnmpOpen $id $ip]
         set txt "SNMPv2 Access Control of $host \[$ip\]:\n"
         append txt "Target Subject Context Privileges Storage \n"
-        $s walk x "aclTarget aclSubject aclResources aclPrivileges \
-		aclStorageType" {
-            set Target     [lindex [lindex $x 0] 2]
-            set Subject    [lindex [lindex $x 1] 2]
-            set Resources  [lindex [lindex $x 2] 2]
-            set Privileges [lindex [lindex $x 3] 2]
-	    set Storage    [lindex [lindex $x 4] 2]
-            append txt [format "%4s  %6s %8s %14s %14s\n" \
+	try {
+	    $s walk x "aclTarget aclSubject aclResources aclPrivileges \
+		    aclStorageType" {
+		set Target     [lindex [lindex $x 0] 2]
+		set Subject    [lindex [lindex $x 1] 2]
+		set Resources  [lindex [lindex $x 2] 2]
+		set Privileges [lindex [lindex $x 3] 2]
+		set Storage    [lindex [lindex $x 4] 2]
+		append txt [format "%4s  %6s %8s %14s %14s\n" \
                         $Target $Subject $Resources $Privileges $Storage]
-        }
-        SnmpClose $s
+	    }
+	} msg {
+            append txt "$msg\n"
+	 }
         writeln $txt
+        $s destroy
+    }
+}
+
+##
+## Dump a complete hierarchy. The user may choose the hierarchy
+## before we start our action. We store the last selected hierarchy
+## in a static variable called dump_mib_tree_path.
+##
+
+proc "Walk MIB Tree" {list} {
+
+    static dump_mib_tree_path
+
+    if {![info exists dump_mib_tree_path]} {
+        set dump_mib_tree_path "mib-2"
+    }
+
+    set path [ined request "Walk MIB Tree:" \
+	        [list [list "MIB path:" $dump_mib_tree_path] ] \
+		[list walk cancel] ]
+
+    if {[lindex $path 0]== "cancel"} return
+
+    set dump_mib_tree_path [lindex $path 1]
+
+    ForeachIpNode id ip host $list {
+	write   "MIB Tree Walk for $host \[$ip\] "
+	writeln "starting at $dump_mib_tree_path:"
+	set s [SnmpOpen $id $ip]
+        SNMP_Walk $s $dump_mib_tree_path
+        $s destroy
+	writeln
     }
 }
 
@@ -564,14 +636,6 @@ proc "Set Parameter" {list} {
     SnmpParameter
 }
 
-##
-## Show the defaults as loaded from the tkined.defaults files.
-##
-
-proc "Show Defaults" {list} {
-    ShowDefaults
-}
-
 ##
 ## Display some help about this tool.
 ##
@@ -584,22 +648,19 @@ proc "Help SNMP-Trouble" {list} {
 	"System Information:" 
 	"    Display the information of the system group." 
 	"" 
-	"Interface -> Status:" 
+	"Interface -> IF Status:" 
 	"    Display status information of the interfaces." 
 	"" 
-	"Interface -> IP Address:" 
-	"    Show which IP address belongs to an interface." 
-	"" 
-	"Interface -> Parameter:" 
+	"Interface -> IF Parameter:" 
 	"    Display interface parameter like speed and MTU." 
 	"" 
-	"Interface -> Usage Statistics:" 
+	"Interface -> IF Usage Statistics:" 
 	"    Display interface statistics." 
 	"" 
-	"Interface -> Error Statistics:" 
+	"Interface -> IF Error Statistics:" 
 	"    Display interface statistics." 
 	"" 
-	"Interface -> Quality:" 
+	"Interface -> IF Quality:" 
 	"    Show the error and discard rate per received packed for" 
 	"    each interface. The output is only valid if your counters" 
 	"    have not wrapped!" 
@@ -608,6 +669,9 @@ proc "Help SNMP-Trouble" {list} {
 	"IP -> IP Statistics" 
 	"    Displays some statistics and parameter of the IP layer." 
 	"" 
+	"IP -> IP Addresses:" 
+	"    Show the IP addresses used by this device." 
+	"" 
 	"IP -> IP Routing Table:" 
 	"    Display the routing table." 
 	"" 
@@ -634,25 +698,12 @@ proc "Help SNMP-Trouble" {list} {
 	"SNMP Statistics:" 
 	"    Statistics and parameter of the SNMP layer." 
 	"" 
+	"Walk MIB Tree:" 
+	"    Walk through the MIB tree and print the object values." 
+	"" 
 	"Set Parameter:" 
 	"    This dialog allows you to set SNMP parameters like retries, " 
 	"    timeouts, community name and port number. " 
-	"" 
-	"Show Defaults:" 
-	"    Show the defaults that may be defined in the tkined.defaults" 
-	"    files. This script makes use of definitions in the form" 
-	"" 
-	"        snmp.community:      <string>" 
-	"        snmp.port:           <number>" 
-	"        snmp.retries:        <number>" 
-	"        snmp.timeout:        <number>" 
-	"" 
-	"        snmp.community.<ip>: <string>" 
-	"        snmp.port.<ip>:      <number>" 
-	"        snmp.retries.<ip>:   <number>" 
-	"        snmp.timeout.<ip>:   <number>" 
-	"" 
-	"    where <ip> is an IP address in dot notation." 
     }
 }
 
@@ -669,13 +720,13 @@ proc "Delete SNMP-Trouble" {list} {
 set menus [ ined create MENU "SNMP-Trouble" \
     "SNMP Devices" "" \
     "System Information" "" \
-    "Interfaces:Status" \
-    "Interfaces:IP Address" \
-    "Interfaces:Parameter" \
-    "Interfaces:Usage Statistics" \
-    "Interfaces:Error Statistics" \
-    "Interfaces:Quality" \
+    "Interfaces:IF Status" \
+    "Interfaces:IF Parameter" \
+    "Interfaces:IF Usage Statistics" \
+    "Interfaces:IF Error Statistics" \
+    "Interfaces:IF Quality" \
     "IP:IP Statistics" \
+    "IP:IP Addresses" \
     "IP:IP Routing Table" \
     "IP:IP ARP Table" \
     "TCP:TCP Statistics" \
@@ -686,7 +737,8 @@ set menus [ ined create MENU "SNMP-Trouble" \
     "SNMP Statistics" "" \
     "SNMPv2:Parties" "SNMPv2:Contexts" "SNMPv2:Views" \
     "SNMPv2:Access Control" "SNMPv2:Statistics" "" \
-    "Set Parameter" "Show Defaults" "" \
+    "Walk MIB Tree" "" \
+    "Set Parameter" "" \
     "Help SNMP-Trouble" "Delete SNMP-Trouble" ]
 
 ##
